'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard   www.joseng.de                                       |
'| Datei: DMX-Sunstrip.bas                                                     |
'| Beschreibung: Firmware fr die 10fach Lampe (Sunstrip) von www.joseng.de    |
'|                                                                             |
'| Prozessor: ATMega 8                                                         |
'| Fuse Bits:  Low Byte: 0xFF    HighByte: 0xD9                                |
'|                                                                             |
'+-----------------------------------------------------------------------------+

$regfile = "m8def.dat"
$crystal = 16000000
$baud = 250000

$hwstack = 50
$swstack = 10
$framesize = 60


'Timer
Config Timer0 = Timer , Prescale = 64
On Timer0 Dimmer_isr
Enable Timer0


'Uart Interrupt Enable
On Urxc Rec_isr
Enable Urxc


'Nulldurchgangserkennung
Enable Int0
On Int0 Int0_int
Config Int0 = Rising


'Ports
Config Portc.0 = Output                                     'Lampe 1
Config Portc.1 = Output                                     'Lampe 2
Config Portc.2 = Output                                     'Lampe 3
Config Portc.3 = Output                                     'Lampe 4
Config Portc.4 = Output                                     'Lampe 5

Config Portb.5 = Output                                     'Lampe 6
Config Portb.4 = Output                                     'Lampe 7
Config Portb.3 = Output                                     'Lampe 8
Config Portb.2 = Output                                     'Lampe 9
Config Portb.1 = Output                                     'Lampe 10

Config Portc.5 = Output                                     'DMX LED
Dmxled Alias Portc.5

Config Portd.3 = Output                                     'DipSwitch Matrix Out 1  (Schalter 1-4)
Config Portd.4 = Output                                     'DipSwitch Matrix Out 2  (Schalter 5-8)
Config Portd.5 = Input                                      'DipSwitch Matrix Input 1 (Schalter 1 & 5)
Config Portd.6 = Input                                      'DipSwitch Matrix Input 2 (Schalter 2 & 6)
Config Portd.7 = Input                                      'DipSwitch Matrix Input 3 (Schalter 3 & 7)
Config Portb.0 = Input                                      'DipSwitch Matrix Input 4 (Schalter 4 & 8)


'Pullup Widerstnde einschalten an Matrix Eingngen
Set Portd.5
Set Portd.6
Set Portd.7
Set Portb.0


'Deklarationen
Dim Dmx_count As Word
Dim Dmx_count_temp As Byte
Dim Dmx_byte As Byte
Dim Dmx_pulse As Byte
Dim Dmx_out As Byte
Dim Dmx_clock As Byte
Dim Netz_pulse As Byte
Dim Dmx_adr As Word
Dim Dmx_adr_temp As Byte
Dim Dmx_adr_c As Word
Dim Dmx_off As Byte                                         'Wird herunter gezhlt, wenn 0 dann geht die DMX-LED aus und zeigt dass kein DMX-Signal kommt

Dim Dmx_adresse As Byte                                     'Hier steht die eingestellte DMX Adresse drin
Const Timervorgabe0 = 246
Dim Dmxwerte(10) As Byte


'Startwerte
Set Dmxled                                                  'DMXLED einschalten
Set Portd.3                                                 'Matrix Out 1
Set Portd.4                                                 'Matrix Out 2
Timer0 = Timervorgabe0                                      'Timer einstellen
Dmx_count = 0
Dmx_clock = 32
'Dmx_adr = 1
Enable Interrupts


'Hauptschleife
Do
  Waitms 10

  'DMX Adresse abfragen
  Gosub Dmx_adresse_abfragen

  Decr Dmx_off
  If Dmx_off = 0 Then
    Reset Dmxled
  End If
Loop
' ******************************************************************************

Rec_isr:                                                    'DMX-Werte einlesen und ablegen
$asm
.def Uart_status = R16
.def Uart_byte = R17
.def Uart_temp1 = R18

      IN Uart_status,USR
      IN Uart_byte,UDR

      SBRC Uart_status,FE
      RJMP  Asm_reset
      RJMP  Asm_Loop1

Asm_reset:
      LDI Uart_temp1,$00
      STS {Dmx_count_temp} , Uart_temp1
      STS {Dmx_byte} , Uart_byte
      RJMP Asm_exit
Asm_loop1:
      STS {Dmx_byte} , Uart_byte
Asm_exit:
$end Asm
   If Dmx_count_temp <> 0 Then
        Incr Dmx_count
           If Dmx_adr_c = Dmx_count Then
               If Dmx_adr_temp < 11 Then
                 Dmxwerte(dmx_adr_temp) = Dmx_byte
                 Incr Dmx_adr_temp
                 Incr Dmx_adr_c
               End If
           End If
   Else
      Dmx_count = 0 : Dmx_count_temp = 1
      Dmx_adr_c = Dmx_adr + 1 : Dmx_adr_temp = 1
      Decr Dmx_clock
      If Dmx_clock = 0 Then : Toggle Dmxled : Dmx_clock = 21 : End If       'Lsst die DMXled bei DMX empfang blinken
   End If
  Dmx_off = 200                                             '2 Sekunden geteilt durch 10 mS
Return


'Dimmerroutine
Dimmer_isr:
  Timer0 = Timervorgabe0

  If Netz_pulse > 0 Then Decr Netz_pulse

  If Netz_pulse <= Dmxwerte(1) Then : Set Portc.0 : Else : Reset Portc.0 : End If
  If Netz_pulse <= Dmxwerte(2) Then : Set Portc.1 : Else : Reset Portc.1 : End If
  If Netz_pulse <= Dmxwerte(3) Then : Set Portc.2 : Else : Reset Portc.2 : End If
  If Netz_pulse <= Dmxwerte(4) Then : Set Portc.3 : Else : Reset Portc.3 : End If
  If Netz_pulse <= Dmxwerte(5) Then : Set Portc.4 : Else : Reset Portc.4 : End If
  If Netz_pulse <= Dmxwerte(6) Then : Set Portb.5 : Else : Reset Portb.5 : End If
  If Netz_pulse <= Dmxwerte(7) Then : Set Portb.4 : Else : Reset Portb.4 : End If
  If Netz_pulse <= Dmxwerte(8) Then : Set Portb.3 : Else : Reset Portb.3 : End If
  If Netz_pulse <= Dmxwerte(9) Then : Set Portb.2 : Else : Reset Portb.2 : End If
  If Netz_pulse <= Dmxwerte(10) Then : Set Portb.1 : Else : Reset Portb.1 : End If
Return


'Nulldurchgangserkennung
Int0_int:
      Netz_pulse = 255
Return


'DMX Adresse vom Dip-Switch einlesen
Dmx_adresse_abfragen:
  Reset Portd.3
  nop                                                       'Ersten (1-4) Dip Schalter auf 0 setzten
  nop
  nop
  If Pind.5 = 0 Then Dmx_adresse = 1                        'Einzelne Dip Switches abfragen
  If Pind.6 = 0 Then Dmx_adresse = Dmx_adresse + 2
  If Pind.7 = 0 Then Dmx_adresse = Dmx_adresse + 4
  If Pinb.0 = 0 Then Dmx_adresse = Dmx_adresse + 8
  Set Portd.3

  Reset Portd.4                                             'Zweiten (5-8) Dip Schalter auf 0 setzten
  nop
  nop
  nop
  If Pind.5 = 0 Then Dmx_adresse = Dmx_adresse + 16         'Einzelne Dip Switches abfragen
  If Pind.6 = 0 Then Dmx_adresse = Dmx_adresse + 32
  If Pind.7 = 0 Then Dmx_adresse = Dmx_adresse + 64
  If Pinb.0 = 0 Then Dmx_adresse = Dmx_adresse + 128
  Set Portd.4

  Dmx_adr = Dmx_adresse
Return